import tensorflow as tf
import pandas as pd
from tensorflow import keras
from keras import backend as K
import numpy as np
import matplotlib.pyplot as plt


def deprocess_image(x):
    x -= x.mean()
    x /= (x.std() + 1e-5)
    x *= 0.1

    x += 255
    x = np.clip(x, 0, 1)

    x *= 255
    x = np.clip(x, 0, 255).astype('uint8')
    return x


def visualise_one_filter(model, layer_name, filter_index, size=150):
    """
    Builds a loss function that maximizes the activation of the nth filter of the specified layer.
    To plot filter image use plt.imshow(visualise_one_filter(model, layer_name, filter_index, size=150))
    :param model: trained model.
    :param layer_name: layer you want to visualise.
    :param filter_index: filter index.
    :param size: size of filter
    :return:
    """
    layer_output = model.get_layer(layer_name).output
    loss = K.mean(layer_output[:, :, :, filter_index])

    grads = K.gradients(loss, model.input)[0]
    grads /= (K.sqrt(K.mean(K.square(grads))) + 1e-5)

    iterate = K.function([model.input], [loss, grads])

    input_img_data = np.random.random((1, size, size, 3)) * 20 + 128

    step = 1
    for i in range(40):
        loss_value, grads_value = iterate([input_img_data])
        input_img_data += grads_value * step

    img = input_img_data[0]
    return deprocess_image(img)


def visualise_filters(model, layer_name, size=64, margin=5):
    results = np.zeros((8 * size + 7 * margin, 8 * size + 7 * margin, 3))
    for i in range(8):
        for j in range(8):
            filter_img = visualise_one_filter(model, layer_name, i + (j * 8), size=size)

            horizontal_start = i * size + i * margin
            horizontal_end = horizontal_start + size
            vertical_start = j * size + j * margin
            vertical_end = vertical_start + size
            results[horizontal_start: horizontal_end, vertical_start: vertical_end, :] = filter_img

    plt.figure(figsize=(20, 20))
    plt.imshow(results)




import pandas as pd
from PIL import Image
import matplotlib.pyplot as plt
import doctest


def check_df_image_size(df: pd.DataFrame, target_column: str) -> None:
    """
    Checks image sizes from Pandas DataFrame and adds two additional columns to it with sizes
    :param df: Pandas dataFrame where you have info.
    :param target_column: Column where paths to images are defined
    :return: None
    """
    col_indx = list(df.columns).index(target_column)
    for i, row in df.iterrows():
        im = Image.open(row[col_indx])
        w, h = im.size
        df.at[i, 'width'] = w
        df.at[i, 'height'] = h
        # df.at[i, 'shape'] = str(f'{w},{h}')


def plot_df_images(df: pd.DataFrame, target_column: str, cols: int, rows: int) -> None:
    """
    Plots images from Pandas DataFrame column where paths are added.
    :param df: Pandas DataFrame with needed info.
    :param target_column: Column where paths are defined.
    :param cols: How many columns with images you want to plot.
    :param rows: How many rows with images you want to plot.
    :return: None
    """
    # TODO add random and len checking
    pictures = df[target_column].tolist()
    fig = plt.figure(figsize=(5 * cols, 5* rows))
    for i in range(1, cols * rows + 1):
        image = pictures[i - 1]
        if image.find('/'):
            img_name = image.split('/')[-1]
        else:
            img_name = image
        fig.add_subplot(rows, cols, i, title=img_name)
        plt.imshow(Image.open(image))
    plt.show()


if __name__ == '__main__':

    plot_df_images(df, target_column, 3, 1)